package de.tudresden.inf.st.mquat.solving.genetic;

import de.tudresden.inf.st.mquat.jastadd.model.Root;
import de.tudresden.inf.st.mquat.jastadd.model.Solution;
import de.tudresden.inf.st.mquat.solving.BenchmarkableSolver;
import de.tudresden.inf.st.mquat.solving.Solver;
import de.tudresden.inf.st.mquat.solving.SolvingException;
import de.tudresden.inf.st.mquat.utils.StopWatch;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

public class GeneticSolver implements BenchmarkableSolver {

  private static final Logger logger = LogManager.getLogger(GeneticSolver.class);

  private Solution lastSolution;
  private long lastSolvingTime;

  private int solutionCounter;

  private StopWatch stopWatch;

  private long maxSolvingTime;
  private boolean timedOut;

  public GeneticSolver() {
    this(Long.MAX_VALUE);
  }

  public GeneticSolver(long maxSolvingTime) {
    this.maxSolvingTime = maxSolvingTime;
    reset();
  }

  @Override
  public Solution solve(Root model) throws SolvingException {
    reset();
    if (model.getNumRequest() == 0) {
      return Solution.emptySolutionOf(model);
    }
    stopWatch = StopWatch.start();

    List<Solution> solutions = new ArrayList<>();

    // do some solvin'

    if (solutions.size() > 0) {
      lastSolution = solutions.get(solutions.size() - 1);
    } else {
      lastSolution = Solution.emptySolutionOf(model);
      logger.warn("Found no solution!");
    }

    lastSolvingTime = stopWatch.time(TimeUnit.MILLISECONDS);

    return lastSolution;
  }

  private void reset() {
    this.lastSolution = null;
    this.solutionCounter = 0;
    this.lastSolvingTime = 0;
    this.timedOut = false;
  }

  @Override
  public String getName() {
    return "genetic";
  }

  @Override
  public long getLastSolvingTime() {
    return lastSolvingTime;
  }

  @Override
  public double getLastObjective() {
    if (lastSolution != null) {
      return lastSolution.computeObjective();
    } else {
      // TODO throw exception or do something reasonable
      return 0d;
    }
  }

  @Override
  public Solver setTimeout(long timeoutValue, TimeUnit timeoutUnit) {
    this.maxSolvingTime = timeoutUnit.toMillis(timeoutValue);
    return this;
  }

  @Override
  public boolean hadTimeout() {
    return this.timedOut;
  }
}
