package de.tudresden.inf.st.mquat.solving.genetic.opt4j;

import de.tudresden.inf.st.mquat.generator.ScenarioDescription;
import de.tudresden.inf.st.mquat.jastadd.model.Root;
import de.tudresden.inf.st.mquat.solving.genetic.opt4j.custom.operator.copy.TreeCopyOperatorModule;
import de.tudresden.inf.st.mquat.solving.genetic.opt4j.custom.operator.crossover.TreeCrossoverOperatorModule;
import de.tudresden.inf.st.mquat.solving.genetic.opt4j.custom.operator.mutate.TreeMutateOperatorModule;
import org.opt4j.core.Individual;
import org.opt4j.core.optimizer.Archive;
import org.opt4j.core.problem.ProblemModule;
import org.opt4j.core.start.Opt4JTask;
import org.opt4j.optimizers.ea.EvolutionaryAlgorithmModule;
import org.opt4j.viewer.ViewerModule;

public class Opt4jModule extends ProblemModule {

  private static Root model;

  public static Root getModel() {
    return model;
  }

  public static void setModel(Root model) {
    Opt4jModule.model = model;
  }

  public static void main(String[] args) {

    final ScenarioDescription SCENARIO_DESCRIPTION = new ScenarioDescription(1, 2, 0, 0, 0, 2, 3, 1.5, 1, 1, 0);
    de.tudresden.inf.st.mquat.generator.ScenarioGenerator sc = new de.tudresden.inf.st.mquat.generator.ScenarioGenerator(
        SCENARIO_DESCRIPTION);

    Opt4jModule.setModel(sc.generate());

    EvolutionaryAlgorithmModule ea = new EvolutionaryAlgorithmModule();
    ea.setGenerations(100000);

    // set population size
    ea.setAlpha(100);

    Opt4jModule testModule = new Opt4jModule();

    TreeCrossoverOperatorModule crossover = new TreeCrossoverOperatorModule();
    TreeMutateOperatorModule mutate = new TreeMutateOperatorModule();
    TreeCopyOperatorModule copy = new TreeCopyOperatorModule();

    // testModule.setFunction(JohannesModule.Function.dtlz);
    ViewerModule viewer = new ViewerModule();
    viewer.isCloseOnStop();

    Opt4JTask task = new Opt4JTask(false);
    task.init(ea, testModule, viewer, crossover, mutate, copy);
    try {
      task.execute();
      Archive archive = task.getInstance(Archive.class);
      for (Individual individual : archive) {
        // obtain the phenotype and objective, etc. of each individual
      }
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      task.close();
    }
  }

  @Override
  protected void configure() {
    bindProblem(Opt4jCreator.class, Opt4jDecoder.class, Opt4jEvaluator.class);
  }

  @Override
  protected void config() {
  }

}
